@@ -597,6 +597,12 @@ class Model : public AbstractModel {
      * @return Observable IDs
      */
     virtual std::vector<std::string> getObservableIds() const;
+    
+    /**
+     * @brief Checks whether the defined noise model is gaussian, i.e., the nllh is quadratic
+     * @return boolean flag
+     */
+    virtual bool hasQuadraticLLH() const;
 
     /**
      * @brief Get the timepoint vector.
