@@ -540,16 +540,16 @@ def constructEdataFromDataFrame(
     overwrite_presim = {}
     for par in list(_get_names_or_ids(model, 'FixedParameter', by_id=by_id)):
         if par + '_preeq' in condition.keys() \
-                and not math.isnan(condition[par + '_preeq']):
-            overwrite_preeq[par] = condition[par + '_preeq']
+                and not math.isnan(condition[par + '_preeq'].astype(float)):
+            overwrite_preeq[par] = condition[par + '_preeq'].astype(float)
         if par + '_presim' in condition.keys() \
-                and not math.isnan(condition[par + '_presim']):
-            overwrite_presim[par] = condition[par + '_presim']
+                and not math.isnan(condition[par + '_presim'].astype(float)):
+            overwrite_presim[par] = condition[par + '_presim'].astype(float)
 
     # fill in fixed parameters
     edata.fixedParameters = condition[
         _get_names_or_ids(model, 'FixedParameter', by_id=by_id)
-    ].values
+    ].astype(float).values
 
     # fill in preequilibration parameters
     if any([overwrite_preeq[key] != condition[key] for key in
@@ -596,7 +596,7 @@ def getEdataFromDataFrame(
         by_id: Optional[bool] = False
 ) -> List[amici.amici.ExpData]:
     """
-    Constructs a ExpData instance according to the provided Model and
+    Constructs a ExpData instances according to the provided Model and
     DataFrame.
 
     :param df:
@@ -642,7 +642,9 @@ def getEdataFromDataFrame(
         selected = np.ones((len(df),), dtype=bool)
         for par_label, par in row.iteritems():
             if math.isnan(par):
-                selected = selected & np.isnan(df[par_label].values)
+                selected = selected & np.isnan(
+                    df[par_label].astype(float).values
+                )
             else:
                 selected = selected & (df[par_label] == par)
         edata_df = df[selected]
