@@ -764,6 +764,10 @@ class Model_TPL_MODELNAME : public amici::Model_ODE {
     virtual const std::string getAmiciCommit() const override {
         return "TPL_AMICI_COMMIT_STRING";
     }
+
+    virtual bool hasQuadraticLLH() const override {
+        return TPL_QUADRATIC_LLH;
+    }
 };
 
 
